@@ -11,7 +11,6 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <errno.h>
-#include <libxml/parser.h>
 #include <errno.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -151,7 +150,7 @@ fail:
  * Returns: -1 on failure. NULL on success.
  */
 int set_ccs_logging(xmlDocPtr ldoc){
-  int facility = SYSLOGFACILITY, loglevel = LOG_LEVEL_INFO, global_debug = 0;
+  int facility = SYSLOGFACILITY, loglevel = SYSLOGLEVEL, global_debug = 0;
   char *res = NULL, *error = NULL;
   xmlXPathContextPtr ctx = NULL;
   unsigned int logmode;
@@ -198,10 +197,14 @@ int set_ccs_logging(xmlDocPtr ldoc){
     if(res) {
       loglevel = logsys_priority_id_get (res);
       if (loglevel < 0)
-	loglevel = LOG_LEVEL_INFO;
+	loglevel = SYSLOGLEVEL;
+
+      if(!debug) {
+	if(loglevel == LOG_LEVEL_DEBUG)
+		debug = 1;
 
-      if(!debug)
 	logsys_config_priority_set (loglevel);
+      }
 
       free(res);
       res=NULL;
@@ -248,14 +251,14 @@ int set_ccs_logging(xmlDocPtr ldoc){
     res=NULL;
   }
 
-  res = do_simple_xml_query(ctx, "/cluster/logging/@filename");
+  res = do_simple_xml_query(ctx, "/cluster/logging/@logfile");
   if(res) {
     if(logsys_config_file_set(&error, res))
-      log_printf(LOG_ERR, "filename: unable to open %s for logging\n", res);
+      log_printf(LOG_ERR, "logfile: unable to open %s for logging\n", res);
     free(res);
     res=NULL;
   } else
-      log_printf(LOG_DEBUG, "filename: use default built-in log file: %s\n", LOGDIR "/ccs.log");
+      log_printf(LOG_DEBUG, "logfile: use default built-in log file: %s\n", LOGDIR "/ccs.log");
 
   res = do_simple_xml_query(ctx, "/cluster/logging/@syslog_facility");
   if(res) {
@@ -276,7 +279,7 @@ int set_ccs_logging(xmlDocPtr ldoc){
   }
 
   if(logmode & LOG_MODE_BUFFER_BEFORE_CONFIG) {
-    log_printf(LOG_DEBUG, "CCS logsys config enabled from set_ccs_logging\n");
+    log_printf(LOG_DEBUG, "logsys config enabled from set_ccs_logging\n");
     logmode &= ~LOG_MODE_BUFFER_BEFORE_CONFIG;
     logmode |= LOG_MODE_FLUSH_AFTER_CONFIG;
     logsys_config_mode_set (logmode);
