@@ -1,22 +1,8 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2008 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <getopt.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <string.h>
 #include <sys/time.h>
 #include <unistd.h>
@@ -41,6 +27,7 @@
 #include "commands.h"
 #include "logging.h"
 #include "barrier.h"
+#include "cman.h"
 #include "ais.h"
 
 extern int we_are_a_cluster_member;
