@@ -28,6 +28,7 @@
 #include <openais/service/timer.h>
 #include <openais/totem/aispoll.h>
 #include "list.h"
+#include "cman.h"
 #include "cnxman-socket.h"
 #include "cnxman-private.h"
 #include "daemon.h"
@@ -1064,6 +1065,7 @@ static void ccsd_timer_fn(void *arg)
 		log_printf(LOG_ERR, "Now got CCS information version %d, continuing\n", config_version);
 		config_error = 0;
 		recalculate_quorum(0, 0);
+		notify_listeners(NULL, EVENT_REASON_CONFIG_UPDATE, config_version);
 	}
 }
 
@@ -1255,7 +1257,6 @@ int process_command(struct connection *con, int cmd, char *cmdbuf,
 		break;
 #ifdef DEBUG
 	case CMAN_CMD_DUMP_OBJDB:
-		P_MEMB("CC: Dump_objdb: filename = %s\n", cmdbuf);
 		dumpfile = fopen(cmdbuf, "w+");
 		if (dumpfile)  {
 			global_objdb->object_dump(OBJECT_PARENT_HANDLE, dumpfile);
@@ -1561,6 +1562,7 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 			send_reconfigure(us->node_id, RECONFIG_PARAM_CONFIG_VERSION, config_version);
 		}
 		recalculate_quorum(0, 0);
+		notify_listeners(NULL, EVENT_REASON_CONFIG_UPDATE, config_version);
 	}
 
 
@@ -1718,6 +1720,7 @@ static void do_reconfigure_msg(void *data)
 			openais_timer_add_duration((unsigned long long)ccsd_poll_interval*1000000, NULL,
 						   ccsd_timer_fn, &ccsd_timer);
 		}
+		notify_listeners(NULL, EVENT_REASON_CONFIG_UPDATE, config_version);
 		break;
 	}
 }
@@ -1796,7 +1799,8 @@ static void do_process_transition(int nodeid, char *data, int len)
 
         /* This is for older nodes. If the join_time of the node matches that already stored AND
 	   the node has been down, then we kill it as this must be a rejoin */
-	if (msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
+	if (msg->minor_version == 0 &&
+	    msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
 		/* Don't duplicate messages */
 		if (node->state != NODESTATE_AISONLY) {
 			if (cluster_is_quorate) {
