@@ -61,7 +61,7 @@ LOGSYS_DECLARE_SYSTEM (NULL,
         LOGDIR "/qdisk.log",
         SYSLOGFACILITY);
 
-LOGSYS_DECLARE_SUBSYS ("QDISK", LOG_LEVEL_INFO);
+LOGSYS_DECLARE_SUBSYS ("QDISK", SYSLOGLEVEL);
 
 static void
 int_handler(int sig)
@@ -1062,7 +1062,7 @@ quorum_logout(qd_ctx *ctx)
 static int
 get_logsys_config_data(int *debug)
 {
-	int ccsfd = -1, loglevel = LOG_LEVEL_INFO, facility = SYSLOGFACILITY;
+	int ccsfd = -1, loglevel = SYSLOGLEVEL, facility = SYSLOGFACILITY;
 	char *val = NULL, *error = NULL;
 	unsigned int logmode;
 	int global_debug = 0;
@@ -1106,28 +1106,34 @@ get_logsys_config_data(int *debug)
 		if (ccs_get(ccsfd, "/cluster/logging/logger_subsys[@subsys=\"QDISK\"]/@syslog_level", &val) == 0) {
 			loglevel = logsys_priority_id_get (val);
 			if (loglevel < 0)
-				loglevel = LOG_LEVEL_INFO;
+				loglevel = SYSLOGLEVEL;
 
 			if (!*debug) {
-				*debug = 1;
+				if (loglevel == LOG_LEVEL_DEBUG)
+					*debug = 1;
+
 				logsys_config_priority_set (loglevel);
 			}
 
 			free(val);
+			val = NULL;
 		} else
 		if (ccs_get(ccsfd, "/cluster/quorumd/@log_level", &val) == 0) { /* check backward compat options */
 			loglevel = logsys_priority_id_get (val);
 			if (loglevel < 0)
-				loglevel = LOG_LEVEL_INFO;
+				loglevel = SYSLOGLEVEL;
 
 			log_printf(LOG_ERR, "<quorumd log_level=\"%s\".. option is depracated\n", val);
 
 			if (!*debug) {
-				*debug = 1;
+				if (loglevel == LOG_LEVEL_DEBUG)
+					*debug = 1;
+
 				logsys_config_priority_set (loglevel);
 			}
 
 			free(val);
+			val = NULL;
 		}
 	} else
 		logsys_config_priority_set (LOG_LEVEL_DEBUG);
@@ -1168,13 +1174,13 @@ get_logsys_config_data(int *debug)
 		val = NULL;
 	}
 
-	if (ccs_get(ccsfd, "/cluster/logging/@filename", &val) == 0) {
+	if (ccs_get(ccsfd, "/cluster/logging/@logfile", &val) == 0) {
 		if(logsys_config_file_set(&error, val))
-			log_printf(LOG_ERR, "filename: unable to open %s for logging\n", val);
+			log_printf(LOG_ERR, "logfile: unable to open %s for logging\n", val);
 		free(val);
 		val = NULL;
 	} else
-		log_printf(LOG_DEBUG, "filename: use default built-in log file: %s\n", LOGDIR "/qdisk.log");
+		log_printf(LOG_DEBUG, "logfile: use default built-in log file: %s\n", LOGDIR "/qdisk.log");
 
 	if (ccs_get(ccsfd, "/cluster/logging/@syslog_facility", &val) == 0) {
 		facility = logsys_facility_id_get (val);
@@ -1197,10 +1203,11 @@ get_logsys_config_data(int *debug)
 
 		logsys_config_facility_set ("QDISK", facility);
 		free(val);
+		val = NULL;
 	}
 
 	if(logmode & LOG_MODE_BUFFER_BEFORE_CONFIG) {
-		log_printf(LOG_DEBUG, "QDISK logsys config enabled from get_logsys_config_data\n");
+		log_printf(LOG_DEBUG, "logsys config enabled from get_logsys_config_data\n");
 		logmode &= ~LOG_MODE_BUFFER_BEFORE_CONFIG;
 		logmode |= LOG_MODE_FLUSH_AFTER_CONFIG;
 		logsys_config_mode_set (logmode);
