@@ -14,7 +14,7 @@
 
 LOGSYS_DECLARE_SYSTEM (NULL, LOG_MODE_OUTPUT_STDERR, NULL, SYSLOGFACILITY);
 
-LOGSYS_DECLARE_SUBSYS ("QDISK", LOG_LEVEL_INFO);
+LOGSYS_DECLARE_SUBSYS ("QDISK", SYSLOGLEVEL);
 
 int
 main(int argc, char **argv)
