@@ -121,8 +121,8 @@ static int buf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 		if (error)
 			return error;
 
-		bh_ip = bget(&sdp->buf_list, blkno);
-		memcpy(bh_ip->b_data, bh_log->b_data, sdp->bsize);
+		bh_ip = bget(sdp, blkno);
+		memcpy(bh_ip->b_data, bh_log->b_data, bh_log->b_size);
 
 		check_magic = ((struct gfs2_meta_header *)
 			       (bh_ip->b_data))->mh_magic;
@@ -220,8 +220,8 @@ static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 		if (error)
 			return error;
 
-		bh_ip = bget(&sdp->buf_list, blkno);
-		memcpy(bh_ip->b_data, bh_log->b_data, sdp->bsize);
+		bh_ip = bget(sdp, blkno);
+		memcpy(bh_ip->b_data, bh_log->b_data, bh_log->b_size);
 
 		/* Unescape */
 		if (esc) {
@@ -424,7 +424,6 @@ int replay_journals(struct gfs2_sbd *sdp){
 	inode_put(sdp->master_dir, not_updated);
 	inode_put(sdp->md.jiinode, not_updated);
 	/* Sync the buffers to disk so we get a fresh start. */
-	bsync(&sdp->buf_list);
-	bsync(&sdp->nvbuf_list);
+	bsync(sdp);
 	return 0;
 }
