@@ -106,15 +106,11 @@ struct gfs2_buffer_head {
 	unsigned int b_count;
 	uint64_t b_blocknr;
 	char *b_data;
+	unsigned int b_size;
 
 	int b_changed;
 };
 
-struct special_blocks {
-	osi_list_t list;
-	uint64_t block;
-};
-
 struct gfs2_sbd;
 struct gfs2_inode {
 	struct gfs2_dinode i_di;
@@ -161,15 +157,6 @@ struct master_dir
 	struct per_node *pn;              /* Array of per_node entries */
 };
 
-struct buf_list {
-	unsigned int num_bufs;
-	unsigned int spills;
-	uint32_t limit;
-	osi_list_t list;
-	struct gfs2_sbd *sbp;
-	osi_list_t buf_hash[BUF_HASH_SIZE];
-};
-
 struct gfs2_sbd {
 	struct gfs2_sb sd_sb;    /* a copy of the ondisk structure */
 	char lockproto[GFS2_LOCKNAME_LEN];
@@ -230,22 +217,18 @@ struct gfs2_sbd {
 
 	unsigned int orig_journals;
 
-	struct buf_list buf_list;   /* transient buffer list */
-	struct buf_list nvbuf_list; /* non-volatile buffer list */
+	unsigned int num_bufs;
+	osi_list_t buf_list;
+	osi_list_t buf_hash[BUF_HASH_SIZE];
 
 	struct gfs2_inode *master_dir;
 	struct master_dir md;
 
+	unsigned int spills;
 	unsigned int writes;
 	int metafs_fd;
+	int metafs_mounted; /* If metafs was already mounted */
 	char metafs_path[PATH_MAX]; /* where metafs is mounted */
-	struct special_blocks bad_blocks;
-	struct special_blocks dup_blocks;
-	struct special_blocks eattr_blocks;
-};
-
-struct metapath {
-	unsigned int mp_list[GFS2_MAX_META_HEIGHT];
 };
 
 extern char *prog_name;
@@ -294,44 +277,26 @@ void gfs2_bitmap_destroy(struct gfs2_bmap *bmap);
 uint64_t gfs2_bitmap_size(struct gfs2_bmap *bmap);
 
 /* block_list.c */
-#define FREE	        (0x0)  /*   0000 */
-#define BLOCK_IN_USE    (0x1)  /*   0001 */
-#define DIR_INDIR_BLK   (0x2)  /*   0010 */
-#define DIR_INODE       (0x3)  /*   0011 */
-#define FILE_INODE      (0x4)  /*   0100 */
-#define LNK_INODE       (0x5)
-#define BLK_INODE       (0x6)
-#define CHR_INODE       (0x7)
-#define FIFO_INODE      (0x8)
-#define SOCK_INODE      (0x9)
-#define DIR_LEAF_INODE  (0xA)  /*   1010 */
-#define JOURNAL_BLK     (0xB)  /*   1011 */
-#define OTHER_META      (0xC)  /*   1100 */
-#define EATTR_META      (0xD)  /*   1101 */
-#define UNUSED1         (0xE)  /*   1110 */
-#define INVALID_META    (0xF)  /*   1111 */
-
 /* Must be kept in sync with mark_to_bitmap array in block_list.c */
 enum gfs2_mark_block {
-	gfs2_block_free = FREE,
-	gfs2_block_used = BLOCK_IN_USE,
-	gfs2_indir_blk = DIR_INDIR_BLK,
-	gfs2_inode_dir = DIR_INODE,
-	gfs2_inode_file = FILE_INODE,
-	gfs2_inode_lnk = LNK_INODE,
-	gfs2_inode_blk = BLK_INODE,
-	gfs2_inode_chr = CHR_INODE,
-	gfs2_inode_fifo = FIFO_INODE,
-	gfs2_inode_sock = SOCK_INODE,
-	gfs2_leaf_blk = DIR_LEAF_INODE,
-	gfs2_journal_blk = JOURNAL_BLK,
-	gfs2_meta_other = OTHER_META,
-	gfs2_meta_eattr = EATTR_META,
-	gfs2_meta_unused = UNUSED1,
-	gfs2_meta_inval = INVALID_META,
-	gfs2_bad_block,      /* Contains at least one bad block */
-	gfs2_dup_block,      /* Contains at least one duplicate block */
-	gfs2_eattr_block,    /* Contains an eattr */
+        gfs2_block_free = 0,
+        gfs2_block_used,
+        gfs2_indir_blk,
+        gfs2_inode_dir,
+        gfs2_inode_file,
+        gfs2_inode_lnk,
+        gfs2_inode_blk,
+        gfs2_inode_chr,
+        gfs2_inode_fifo,
+        gfs2_inode_sock,
+        gfs2_leaf_blk,
+        gfs2_journal_blk,
+        gfs2_meta_other,
+        gfs2_meta_eattr,
+        gfs2_meta_inval = 15,
+        gfs2_bad_block,      /* Contains at least one bad block */
+        gfs2_dup_block,      /* Contains at least one duplicate block */
+        gfs2_eattr_block,    /* Contains an eattr */
 };
 
 struct gfs2_block_query {
@@ -357,37 +322,34 @@ struct gfs2_block_list {
         union gfs2_block_lists list;
 };
 
-struct gfs2_block_list *gfs2_block_list_create(struct gfs2_sbd *sdp,
-					       uint64_t size,
+struct gfs2_block_list *gfs2_block_list_create(uint64_t size,
 					       uint64_t *addl_mem_needed);
-struct special_blocks *blockfind(struct special_blocks *blist, uint64_t num);
-void gfs2_special_set(struct special_blocks *blocklist, uint64_t block);
-void gfs2_special_clear(struct special_blocks *blocklist, uint64_t block);
-void gfs2_special_free(struct special_blocks *blist);
-int gfs2_block_mark(struct gfs2_sbd *sdp, struct gfs2_block_list *il,
-		    uint64_t block, enum gfs2_mark_block mark);
-int gfs2_block_set(struct gfs2_sbd *sdp, struct gfs2_block_list *il,
-		   uint64_t block, enum gfs2_mark_block mark);
-int gfs2_block_clear(struct gfs2_sbd *sdp, struct gfs2_block_list *il,
-		     uint64_t block, enum gfs2_mark_block m);
-int gfs2_block_check(struct gfs2_sbd *sdp, struct gfs2_block_list *il,
-		     uint64_t block, struct gfs2_block_query *val);
-void *gfs2_block_list_destroy(struct gfs2_sbd *sdp,
-			      struct gfs2_block_list *il);
+int gfs2_block_mark(struct gfs2_block_list *il, uint64_t block,
+					enum gfs2_mark_block mark);
+int gfs2_block_set(struct gfs2_block_list *il, uint64_t block,
+				   enum gfs2_mark_block mark);
+int gfs2_block_clear(struct gfs2_block_list *il, uint64_t block,
+					 enum gfs2_mark_block m);
+int gfs2_block_check(struct gfs2_block_list *il, uint64_t block,
+					 struct gfs2_block_query *val);
+int gfs2_block_check_for_mark(struct gfs2_block_list *il, uint64_t block,
+			      enum gfs2_mark_block mark);
+void *gfs2_block_list_destroy(struct gfs2_block_list *il);
+int gfs2_find_next_block_type(struct gfs2_block_list *il,
+			      enum gfs2_mark_block m, uint64_t *b);
 
 /* buf.c */
-void init_buf_list(struct gfs2_sbd *sdp, struct buf_list *bl, uint32_t limit);
-struct gfs2_buffer_head *bfind(struct buf_list *bl, uint64_t num);
-struct gfs2_buffer_head *bget_generic(struct buf_list *bl, uint64_t num,
+struct gfs2_buffer_head *bget_generic(struct gfs2_sbd *sdp, uint64_t num,
 				      int find_existing, int read_disk);
-struct gfs2_buffer_head *bget(struct buf_list *bl, uint64_t num);
-struct gfs2_buffer_head *bread(struct buf_list *bl, uint64_t num);
-struct gfs2_buffer_head *bget_zero(struct buf_list *bl, uint64_t num);
+struct gfs2_buffer_head *bget(struct gfs2_sbd *sdp, uint64_t num);
+struct gfs2_buffer_head *bread(struct gfs2_sbd *sdp, uint64_t num);
+struct gfs2_buffer_head *bget_zero(struct gfs2_sbd *sdp, uint64_t num);
 struct gfs2_buffer_head *bhold(struct gfs2_buffer_head *bh);
 void brelse(struct gfs2_buffer_head *bh, enum update_flags updated);
-void bsync(struct buf_list *bl);
-void bcommit(struct buf_list *bl);
-void bcheck(struct buf_list *bl);
+void bsync(struct gfs2_sbd *sdp);
+void bcommit(struct gfs2_sbd *sdp);
+void bcheck(struct gfs2_sbd *sdp);
+void write_buffer(struct gfs2_sbd *sdp, struct gfs2_buffer_head *bh);
 
 /* device_geometry.c */
 void device_geometry(struct gfs2_sbd *sdp);
@@ -425,17 +387,6 @@ void build_rgrps(struct gfs2_sbd *sdp, int write);
 #define IS_LEAF     (1)
 #define IS_DINODE   (2)
 
-static __inline__ uint64_t *
-metapointer(struct gfs2_buffer_head *bh, unsigned int height,
-	    struct metapath *mp)
-{
-	unsigned int head_size = (height > 0) ?
-		sizeof(struct gfs2_meta_header) : sizeof(struct gfs2_dinode);
-
-	return ((uint64_t *)(bh->b_data + head_size)) + mp->mp_list[height];
-}
-
-struct metapath *find_metapath(struct gfs2_inode *ip, uint64_t block);
 struct gfs2_inode *inode_get(struct gfs2_sbd *sdp,
 			     struct gfs2_buffer_head *bh);
 void inode_put(struct gfs2_inode *ip, enum update_flags updated);
@@ -469,7 +420,7 @@ void block_map(struct gfs2_inode *ip, uint64_t lblock, int *new,
 void gfs2_get_leaf_nr(struct gfs2_inode *dip, uint32_t index,
 					  uint64_t *leaf_out);
 void gfs2_put_leaf_nr(struct gfs2_inode *dip, uint32_t inx, uint64_t leaf_out);
-void gfs2_free_block(struct gfs2_sbd *sdp, uint64_t block);
+
 int gfs2_freedi(struct gfs2_sbd *sdp, uint64_t block);
 int gfs2_get_leaf(struct gfs2_inode *dip, uint64_t leaf_no,
 				  struct gfs2_buffer_head **bhp);
@@ -492,71 +443,6 @@ void write_journal(struct gfs2_sbd *sdp, struct gfs2_inode *ip, unsigned int j,
 
 int device_size(int fd, uint64_t *bytes);
 
-/* gfs1.c - GFS1 backward compatibility functions */
-struct gfs_indirect {
-	struct gfs2_meta_header in_header;
-
-	char in_reserved[64];
-};
-
-struct gfs_dinode {
-	struct gfs2_meta_header di_header;
-
-	struct gfs2_inum di_num; /* formal inode # and block address */
-
-	uint32_t di_mode;	/* mode of file */
-	uint32_t di_uid;	/* owner's user id */
-	uint32_t di_gid;	/* owner's group id */
-	uint32_t di_nlink;	/* number (qty) of links to this file */
-	uint64_t di_size;	/* number (qty) of bytes in file */
-	uint64_t di_blocks;	/* number (qty) of blocks in file */
-	int64_t di_atime;	/* time last accessed */
-	int64_t di_mtime;	/* time last modified */
-	int64_t di_ctime;	/* time last changed */
-
-	/*  Non-zero only for character or block device nodes  */
-	uint32_t di_major;	/* device major number */
-	uint32_t di_minor;	/* device minor number */
-
-	/*  Block allocation strategy  */
-	uint64_t di_rgrp;	/* dinode rgrp block number */
-	uint64_t di_goal_rgrp;	/* rgrp to alloc from next */
-	uint32_t di_goal_dblk;	/* data block goal */
-	uint32_t di_goal_mblk;	/* metadata block goal */
-
-	uint32_t di_flags;	/* GFS_DIF_... */
-
-	/*  struct gfs_rindex, struct gfs_jindex, or struct gfs_dirent */
-	uint32_t di_payload_format;  /* GFS_FORMAT_... */
-	uint16_t di_type;	/* GFS_FILE_... type of file */
-	uint16_t di_height;	/* height of metadata (0 == stuffed) */
-	uint32_t di_incarn;	/* incarnation (unused, see gfs_meta_header) */
-	uint16_t di_pad;
-
-	/*  These only apply to directories  */
-	uint16_t di_depth;	/* Number of bits in the table */
-	uint32_t di_entries;	/* The # (qty) of entries in the directory */
-
-	/*  This formed an on-disk chain of unused dinodes  */
-	struct gfs2_inum di_next_unused;  /* used in old versions only */
-
-	uint64_t di_eattr;	/* extended attribute block number */
-
-	char di_reserved[56];
-};
-
-void gfs1_lookup_block(struct gfs2_inode *ip, struct gfs2_buffer_head *bh,
-		  unsigned int height, struct metapath *mp,
-		       int create, int *new, uint64_t *block);
-void gfs1_block_map(struct gfs2_inode *ip, uint64_t lblock, int *new,
-		    uint64_t *dblock, uint32_t *extlen, int prealloc);
-int gfs1_readi(struct gfs2_inode *ip, void *buf, uint64_t offset,
-	       unsigned int size);
-int gfs1_rindex_read(struct gfs2_sbd *sdp, int fd, int *count1);
-int gfs1_ri_update(struct gfs2_sbd *sdp, int fd, int *rgcount);
-struct gfs2_inode *gfs_inode_get(struct gfs2_sbd *sdp,
-				 struct gfs2_buffer_head *bh);
-
 /* locking.c */
 void test_locking(char *lockproto, char *locktable);
 
@@ -631,13 +517,12 @@ int gfs2_query(int *setonabort, struct gfs2_options *opts,
 /* misc.c */
 #define SYS_BASE "/sys/fs/gfs2"
 
-uint32_t compute_heightsize(struct gfs2_sbd *sdp, uint64_t *heightsize,
-			    uint32_t bsize1, int diptrs, int inptrs);
 void compute_constants(struct gfs2_sbd *sdp);
 int find_gfs2_meta(struct gfs2_sbd *sdp);
 int dir_exists(const char *dir);
 void check_for_gfs2(struct gfs2_sbd *sdp);
 void mount_gfs2_meta(struct gfs2_sbd *sdp);
+void lock_for_admin(struct gfs2_sbd *sdp);
 void cleanup_metafs(struct gfs2_sbd *sdp);
 char *get_list(void);
 char **str2lines(char *str);
