@@ -344,7 +344,10 @@ static void node_history_fail(struct lockspace *ls, int nodeid,
 	if (cfgd_enable_fencing && !node->add_time)
 		node->check_fencing = 1;
 
-	if (cfgd_enable_quorum)
+	/* fenced will take care of making sure the quorum value
+	   is adjusted for all the failures */
+
+	if (cfgd_enable_quorum && !cfgd_enable_fencing)
 		node->check_quorum = 1;
 
 	node->check_fs = 1;
@@ -405,8 +408,6 @@ static int check_fencing_done(struct lockspace *ls)
 	return 1;
 }
 
-/* wait for cman to see all the same nodes failed, and to say there's quorum */
-
 static int check_quorum_done(struct lockspace *ls)
 {
 	struct node *node;
@@ -415,10 +416,9 @@ static int check_quorum_done(struct lockspace *ls)
 	if (!cfgd_enable_quorum)
 		return 1;
 
-	if (!cman_quorate) {
-		log_group(ls, "check_quorum %d", cman_quorate);
-		return 0;
-	}
+	/* wait for cman to see all the same nodes failed, so we know that
+	   cman_quorate is adjusted for the same failures we've seen
+	   (see comment in fenced about the assumption here) */
 
 	list_for_each_entry(node, &ls->node_history, list) {
 		if (!node->check_quorum)
@@ -436,6 +436,11 @@ static int check_quorum_done(struct lockspace *ls)
 	if (wait_count)
 		return 0;
 
+	if (!cman_quorate) {
+		log_group(ls, "check_quorum not quorate");
+		return 0;
+	}
+
 	log_group(ls, "check_quorum done");
 	return 1;
 }
@@ -528,6 +533,10 @@ static void start_kernel(struct lockspace *ls)
 	log_group(ls, "start_kernel %u member_count %d",
 		  cg->seq, cg->member_count);
 
+	/* needs to happen before setting control which starts recovery */
+	if (ls->joining)
+		set_sysfs_id(ls->name, ls->global_id);
+
 	format_member_ids(ls);
 	format_renew_ids(ls);
 	set_configfs_members(ls->name, member_count, member_ids,
@@ -536,7 +545,6 @@ static void start_kernel(struct lockspace *ls)
 	ls->kernel_stopped = 0;
 
 	if (ls->joining) {
-		set_sysfs_id(ls->name, ls->global_id);
 		set_sysfs_event_done(ls->name, 0);
 		ls->joining = 0;
 	}
