@@ -25,6 +25,8 @@
 #include "config.h"
 #include "ccs.h"
 
+static int ccs_handle;
+
 /* was a config value set on command line?, 0 or 1.
    optk is a kernel option, optd is a daemon option */
 
@@ -51,27 +53,13 @@ int cfgd_drop_resources_time	= DEFAULT_DROP_RESOURCES_TIME;
 int cfgd_drop_resources_count	= DEFAULT_DROP_RESOURCES_COUNT;
 int cfgd_drop_resources_age	= DEFAULT_DROP_RESOURCES_AGE;
 
-
-static int open_ccs(void)
-{
-	int i = 0, cd;
-
-	while ((cd = ccs_connect()) < 0) {
-		sleep(1);
-		if (++i > 9 && !(i % 10))
-			log_error("connect to ccs error %d, "
-				  "check ccsd or cluster status", cd);
-	}
-	return cd;
-}
-
-static void read_ccs_int(int cd, char *path, int *config_val)
+static void read_ccs_int(char *path, int *config_val)
 {
 	char *str;
 	int val;
 	int error;
 
-	error = ccs_get(cd, path, &str);
+	error = ccs_get(ccs_handle, path, &str);
 	if (error || !str)
 		return;
 
@@ -87,49 +75,6 @@ static void read_ccs_int(int cd, char *path, int *config_val)
 	free(str);
 }
 
-#define GROUPD_COMPAT_PATH "/cluster/group/@groupd_compat"
-#define ENABLE_WITHDRAW_PATH "/cluster/gfs_controld/@enable_withdraw"
-#define ENABLE_PLOCK_PATH "/cluster/gfs_controld/@enable_plock"
-#define PLOCK_DEBUG_PATH "/cluster/gfs_controld/@plock_debug"
-#define PLOCK_RATE_LIMIT_PATH "/cluster/gfs_controld/@plock_rate_limit"
-#define PLOCK_OWNERSHIP_PATH "/cluster/gfs_controld/@plock_ownership"
-#define DROP_RESOURCES_TIME_PATH "/cluster/gfs_controld/@drop_resources_time"
-#define DROP_RESOURCES_COUNT_PATH "/cluster/gfs_controld/@drop_resources_count"
-#define DROP_RESOURCES_AGE_PATH "/cluster/gfs_controld/@drop_resources_age"
-
-/* These config values are set from cluster.conf only if they haven't already
-   been set on the command line. */
-
-void read_ccs(void)
-{
-	int cd;
-
-	cd = open_ccs();
-	if (cd < 0)
-		return;
-
-	if (!optd_groupd_compat)
-		read_ccs_int(cd, GROUPD_COMPAT_PATH, &cfgd_groupd_compat);
-	if (!optd_enable_withdraw)
-		read_ccs_int(cd, ENABLE_WITHDRAW_PATH, &cfgd_enable_withdraw);
-	if (!optd_enable_plock)
-		read_ccs_int(cd, ENABLE_PLOCK_PATH, &cfgd_enable_plock);
-	if (!optd_plock_debug)
-		read_ccs_int(cd, PLOCK_DEBUG_PATH, &cfgd_plock_debug);
-	if (!optd_plock_rate_limit)
-		read_ccs_int(cd, PLOCK_RATE_LIMIT_PATH, &cfgd_plock_rate_limit);
-	if (!optd_plock_ownership)
-		read_ccs_int(cd, PLOCK_OWNERSHIP_PATH, &cfgd_plock_ownership);
-	if (!optd_drop_resources_time)
-		read_ccs_int(cd, DROP_RESOURCES_TIME_PATH, &cfgd_drop_resources_time);
-	if (!optd_drop_resources_count)
-		read_ccs_int(cd, DROP_RESOURCES_COUNT_PATH, &cfgd_drop_resources_count);
-	if (!optd_drop_resources_age)
-		read_ccs_int(cd, DROP_RESOURCES_AGE_PATH, &cfgd_drop_resources_age);
-
-	ccs_disconnect(cd);
-}
-
 #define LOCKSPACE_NODIR "/cluster/dlm/lockspace[@name=\"%s\"]/@nodir"
 
 void read_ccs_nodir(struct mountgroup *mg, char *buf)
@@ -138,16 +83,11 @@ void read_ccs_nodir(struct mountgroup *mg, char *buf)
 	char *str;
 	int val;
 	int error;
-	int cd;
-
-	cd = open_ccs();
-	if (cd < 0)
-		return;
 
 	memset(path, 0, PATH_MAX);
 	sprintf(path, LOCKSPACE_NODIR, mg->name);
 
-	error = ccs_get(cd, path, &str);
+	error = ccs_get(ccs_handle, path, &str);
 	if (error || !str)
 		return;
 
@@ -162,7 +102,58 @@ void read_ccs_nodir(struct mountgroup *mg, char *buf)
 
 	log_debug("%s is %u", path, val);
 	free(str);
+}
 
-	ccs_disconnect(cd);
+#define GROUPD_COMPAT_PATH "/cluster/group/@groupd_compat"
+#define ENABLE_WITHDRAW_PATH "/cluster/gfs_controld/@enable_withdraw"
+#define ENABLE_PLOCK_PATH "/cluster/gfs_controld/@enable_plock"
+#define PLOCK_DEBUG_PATH "/cluster/gfs_controld/@plock_debug"
+#define PLOCK_RATE_LIMIT_PATH "/cluster/gfs_controld/@plock_rate_limit"
+#define PLOCK_OWNERSHIP_PATH "/cluster/gfs_controld/@plock_ownership"
+#define DROP_RESOURCES_TIME_PATH "/cluster/gfs_controld/@drop_resources_time"
+#define DROP_RESOURCES_COUNT_PATH "/cluster/gfs_controld/@drop_resources_count"
+#define DROP_RESOURCES_AGE_PATH "/cluster/gfs_controld/@drop_resources_age"
+
+int setup_ccs(void)
+{
+	int i = 0, cd;
+
+        while ((cd = ccs_connect()) < 0) {
+                sleep(1);
+                if (++i > 9 && !(i % 10))
+                        log_error("connect to ccs error %d, "
+                                  "check cluster status", cd);
+        }
+
+        ccs_handle = cd;
+
+	/* These config values are set from cluster.conf only if they haven't
+	   already been set on the command line. */
+
+	if (!optd_groupd_compat)
+		read_ccs_int(GROUPD_COMPAT_PATH, &cfgd_groupd_compat);
+	if (!optd_enable_withdraw)
+		read_ccs_int(ENABLE_WITHDRAW_PATH, &cfgd_enable_withdraw);
+	if (!optd_enable_plock)
+		read_ccs_int(ENABLE_PLOCK_PATH, &cfgd_enable_plock);
+	if (!optd_plock_debug)
+		read_ccs_int(PLOCK_DEBUG_PATH, &cfgd_plock_debug);
+	if (!optd_plock_rate_limit)
+		read_ccs_int(PLOCK_RATE_LIMIT_PATH, &cfgd_plock_rate_limit);
+	if (!optd_plock_ownership)
+		read_ccs_int(PLOCK_OWNERSHIP_PATH, &cfgd_plock_ownership);
+	if (!optd_drop_resources_time)
+		read_ccs_int(DROP_RESOURCES_TIME_PATH, &cfgd_drop_resources_time);
+	if (!optd_drop_resources_count)
+		read_ccs_int(DROP_RESOURCES_COUNT_PATH, &cfgd_drop_resources_count);
+	if (!optd_drop_resources_age)
+		read_ccs_int(DROP_RESOURCES_AGE_PATH, &cfgd_drop_resources_age);
+
+	return 0;
+}
+
+void close_ccs(void)
+{
+	ccs_disconnect(ccs_handle);
 }
 
