@@ -297,8 +297,7 @@ static void process_uevent(int ci)
 		if (lock_module)
 			return;
 
-		if (group_mode == GROUP_LIBGROUP)
-			do_leave(argv[3], 0);
+		do_leave(argv[3], 0);
 
 	} else if (!strcmp(act, "change@")) {
 		if (!lock_module)
@@ -315,6 +314,8 @@ static void process_uevent(int ci)
 
 		if (group_mode == GROUP_LIBGROUP)
 			do_withdraw_old(argv[3]);
+		else
+			log_error("TODO withdraw for libcpg");
 
 	} else {
 		if (!lock_module)
@@ -442,6 +443,122 @@ static void do_reply(int fd, int cmd, char *name, int result, void *buf,
 	free(reply);
 }
 
+static void query_mountgroup_info(int fd, char *name)
+{
+	struct mountgroup *mg;
+	struct gfsc_mountgroup mountgroup;
+	int rv;
+
+	mg = find_mg(name);
+	if (!mg) {
+		rv = -ENOENT;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_mountgroup_info_group(mg, &mountgroup);
+	else
+		rv = set_mountgroup_info(mg, &mountgroup);
+ out:
+	do_reply(fd, GFSC_CMD_MOUNTGROUP_INFO, name, rv,
+		 (char *)&mountgroup, sizeof(mountgroup));
+}
+
+static void query_node_info(int fd, char *name, int nodeid)
+{
+	struct mountgroup *mg;
+	struct gfsc_node node;
+	int rv;
+
+	mg = find_mg(name);
+	if (!mg) {
+		rv = -ENOENT;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_node_info_group(mg, nodeid, &node);
+	else
+		rv = set_node_info(mg, nodeid, &node);
+ out:
+	do_reply(fd, GFSC_CMD_NODE_INFO, name, rv,
+		 (char *)&node, sizeof(node));
+}
+
+static void query_mountgroups(int fd, int max)
+{
+	int mg_count = 0;
+	struct gfsc_mountgroup *mgs = NULL;
+	int rv, result;
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_mountgroups_group(&mg_count, &mgs);
+	else
+		rv = set_mountgroups(&mg_count, &mgs);
+
+	if (rv < 0) {
+		result = rv;
+		mg_count = 0;
+		goto out;
+	}
+
+	if (mg_count > max) {
+		result = -E2BIG;
+		mg_count = max;
+	} else {
+		result = mg_count;
+	}
+ out:
+	do_reply(fd, GFSC_CMD_MOUNTGROUPS, NULL, result,
+		 (char *)mgs, mg_count * sizeof(struct gfsc_mountgroup));
+
+	if (mgs)
+		free(mgs);
+}
+
+static void query_mountgroup_nodes(int fd, char *name, int option, int max)
+{
+	struct mountgroup *mg;
+	int node_count = 0;
+	struct gfsc_node *nodes = NULL;
+	int rv, result;
+
+	mg = find_mg(name);
+	if (!mg) {
+		result = -ENOENT;
+		node_count = 0;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_mountgroup_nodes_group(mg, option, &node_count, &nodes);
+	else
+		rv = set_mountgroup_nodes(mg, option, &node_count, &nodes);
+
+	if (rv < 0) {
+		result = rv;
+		node_count = 0;
+		goto out;
+	}
+
+	/* node_count is the number of structs copied/returned; the caller's
+	   max may be less than that, in which case we copy as many as they
+	   asked for and return -E2BIG */
+
+	if (node_count > max) {
+		result = -E2BIG;
+		node_count = max;
+	} else {
+		result = node_count;
+	}
+ out:
+	do_reply(fd, GFSC_CMD_MOUNTGROUP_NODES, name, result,
+		 (char *)nodes, node_count * sizeof(struct gfsc_node));
+
+	if (nodes)
+		free(nodes);
+}
+
 void client_reply_remount(struct mountgroup *mg, int result)
 {
 	struct gfsc_mount_args *ma = &mg->mount_args;
@@ -725,6 +842,7 @@ void process_connection(int ci)
 		if (group_mode == GROUP_LIBGROUP)
 			remount_mountgroup_old(ci, ma);
 #if 0
+		/* FIXME */
 		else
 			remount_mountgroup(ci, ma);
 #endif
@@ -837,6 +955,18 @@ static void *process_queries(void *arg)
 		case GFSC_CMD_DUMP_PLOCKS:
 			query_dump_plocks(f, h.name);
 			break;
+		case GFSC_CMD_MOUNTGROUP_INFO:
+			query_mountgroup_info(f, h.name);
+			break;
+		case GFSC_CMD_NODE_INFO:
+			query_node_info(f, h.name, h.data);
+			break;
+		case GFSC_CMD_MOUNTGROUPS:
+			query_mountgroups(f, h.data);
+			break;
+		case GFSC_CMD_MOUNTGROUP_NODES:
+			query_mountgroup_nodes(f, h.name, h.option, h.data);
+			break;
 		default:
 			break;
 		}
@@ -867,10 +997,10 @@ static int setup_queries(void)
 	return 0;
 }
 
-static void cluster_dead(int ci)
+void cluster_dead(int ci)
 {
 	log_error("cluster is down, exiting");
-	exit(1);
+	daemon_quit = 1;
 }
 
 static void dead_dlmcontrol(int ci)
@@ -878,7 +1008,7 @@ static void dead_dlmcontrol(int ci)
 	log_error("dlm_controld poll error %x", pollfd[ci].revents);
 }
 
-static int loop(void)
+static void loop(void)
 {
 	int poll_timeout = -1;
 	int rv, i;
@@ -897,16 +1027,20 @@ static int loop(void)
 		goto out;
 	client_add(rv, process_listener, NULL);
 
-	rv = setup_uevent();
-	if (rv < 0)
-		goto out;
-	client_add(rv, process_uevent, NULL);
-
 	rv = setup_cman();
 	if (rv < 0)
 		goto out;
 	client_add(rv, process_cman, cluster_dead);
 
+	rv = setup_ccs();
+	if (rv < 0)
+		goto out;
+
+	rv = setup_uevent();
+	if (rv < 0)
+		goto out;
+	client_add(rv, process_uevent, NULL);
+
 	group_mode = GROUP_LIBCPG;
 
 	if (cfgd_groupd_compat) {
@@ -962,9 +1096,8 @@ static int loop(void)
 	for (;;) {
 		rv = poll(pollfd, client_maxi + 1, poll_timeout);
 		if (rv == -1 && errno == EINTR) {
-			if (daemon_quit && list_empty(&mountgroups)) {
-				exit(1);
-			}
+			if (daemon_quit && list_empty(&mountgroups))
+				goto out;
 			daemon_quit = 0;
 			continue;
 		}
@@ -989,6 +1122,9 @@ static int loop(void)
 			}
 		}
 
+		if (daemon_quit)
+			break;
+
 		poll_timeout = -1;
 
 		if (poll_dlm) {
@@ -1019,9 +1155,14 @@ static int loop(void)
 
 		query_unlock();
 	}
-	rv = 0;
  out:
-	return rv;
+	if (cfgd_groupd_compat)
+		close_groupd();
+	close_ccs();
+	close_cman();
+
+	if (!list_empty(&mountgroups))
+		log_error("mountgroups abandoned");
 }
 
 static void lockfile(void)
@@ -1237,12 +1378,12 @@ int main(int argc, char **argv)
 	openlog("gfs_controld", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
-	read_ccs();
-
 	set_scheduler();
 	set_oom_adj(-16);
 
-	return loop();
+	loop();
+
+	return 0;
 }
 
 void daemon_dump_save(void)
